'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _camelCase = require('lodash/camelCase');

var _camelCase2 = _interopRequireDefault(_camelCase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Recursively initialize a components hierarchy
 * @param  {Object|Function} component             [Function if a leaf component is supplied, Object if a wrapper component is detected]
 * @param  {Object}          styles                [Global styles object]
 * @param  {Object}          parentLayerComponents [An object of already initialized components from parent layer]
 * @param  {Array}           keyPath               [Path to current component]
 * @return {Object}                                [An object containing initialized component or components hierarchy]
 */
var initComponent = function initComponent(component, styles, parentLayerComponents) {
  var keyPath = arguments.length <= 3 || arguments[3] === undefined ? [] : arguments[3];

  if (typeof component !== 'function') {
    return Object.keys(component).reduce(function (result, componentKey) {
      return _extends({}, result, _defineProperty({}, componentKey, initComponent(component[componentKey], styles, parentLayerComponents, [].concat(_toConsumableArray(keyPath), [componentKey]))));
    }, {});
  } else {
    return component(styles[(0, _camelCase2.default)(keyPath[0])] || {}, parentLayerComponents);
  }
};

/**
 * Recursive initialization of a layer
 * @param  {Object} layer                 [Components object related to current UI abstraction layer]
 * @param  {Object} styles                [Global styles object]
 * @param  {Object} parentLayerComponents [An object of already initialized components from parent layer]
 * @return {Object}                       [An object containing initialized components for current layer]
 */
var initLayer = function initLayer(layer, styles, parentLayerComponents) {
  return Object.keys(layer).reduce(function (layerComponents, componentKey) {
    return _extends({}, layerComponents, _defineProperty({}, componentKey, initComponent(layer[componentKey], styles, parentLayerComponents, [componentKey])));
  }, {});
};

/**
 * React UI initialization
 * @param  {Object} styles [Styles for every component]
 * @return {Object}        [UI object that contains initialized components]
 */
var init = function init() {
  var styles = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  return function () {
    for (var _len = arguments.length, uiLayers = Array(_len), _key = 0; _key < _len; _key++) {
      uiLayers[_key] = arguments[_key];
    }

    return uiLayers.reduce(function (initializedComponents, layer) {
      return _extends({}, initializedComponents, initLayer(layer, styles, initializedComponents));
    }, {});
  };
};

exports.default = init;